# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from airflow.providers.google_vendor.googleads.v12.enums.types import (
    placement_type as gage_placement_type,
)


__protobuf__ = proto.module(
    package="airflow.providers.google_vendor.googleads.v12.resources",
    marshal="google.ads.googleads.v12",
    manifest={"GroupPlacementView",},
)


class GroupPlacementView(proto.Message):
    r"""A group placement view.

    Attributes:
        resource_name (str):
            Output only. The resource name of the group placement view.
            Group placement view resource names have the form:

            ``customers/{customer_id}/groupPlacementViews/{ad_group_id}~{base64_placement}``
        placement (str):
            Output only. The automatic placement string
            at group level, e. g. web domain, mobile app ID,
            or a YouTube channel ID.

            This field is a member of `oneof`_ ``_placement``.
        display_name (str):
            Output only. Domain name for websites and
            YouTube channel name for YouTube channels.

            This field is a member of `oneof`_ ``_display_name``.
        target_url (str):
            Output only. URL of the group placement, for
            example, domain, link to the mobile application
            in app store, or a YouTube channel URL.

            This field is a member of `oneof`_ ``_target_url``.
        placement_type (google.ads.googleads.v12.enums.types.PlacementTypeEnum.PlacementType):
            Output only. Type of the placement, for
            example, Website, YouTube Channel, Mobile
            Application.
    """

    resource_name = proto.Field(proto.STRING, number=1,)
    placement = proto.Field(proto.STRING, number=6, optional=True,)
    display_name = proto.Field(proto.STRING, number=7, optional=True,)
    target_url = proto.Field(proto.STRING, number=8, optional=True,)
    placement_type = proto.Field(
        proto.ENUM,
        number=5,
        enum=gage_placement_type.PlacementTypeEnum.PlacementType,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from airflow.providers.google_vendor.googleads.v12.enums.types import search_term_targeting_status


__protobuf__ = proto.module(
    package="airflow.providers.google_vendor.googleads.v12.resources",
    marshal="google.ads.googleads.v12",
    manifest={"SearchTermView",},
)


class SearchTermView(proto.Message):
    r"""A search term view with metrics aggregated by search term at
    the ad group level.

    Attributes:
        resource_name (str):
            Output only. The resource name of the search term view.
            Search term view resource names have the form:

            ``customers/{customer_id}/searchTermViews/{campaign_id}~{ad_group_id}~{URL-base64_search_term}``
        search_term (str):
            Output only. The search term.

            This field is a member of `oneof`_ ``_search_term``.
        ad_group (str):
            Output only. The ad group the search term
            served in.

            This field is a member of `oneof`_ ``_ad_group``.
        status (google.ads.googleads.v12.enums.types.SearchTermTargetingStatusEnum.SearchTermTargetingStatus):
            Output only. Indicates whether the search
            term is currently one of your targeted or
            excluded keywords.
    """

    resource_name = proto.Field(proto.STRING, number=1,)
    search_term = proto.Field(proto.STRING, number=5, optional=True,)
    ad_group = proto.Field(proto.STRING, number=6, optional=True,)
    status = proto.Field(
        proto.ENUM,
        number=4,
        enum=search_term_targeting_status.SearchTermTargetingStatusEnum.SearchTermTargetingStatus,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from airflow.providers.google_vendor.googleads.v12.common.types import audiences
from airflow.providers.google_vendor.googleads.v12.enums.types import audience_status


__protobuf__ = proto.module(
    package="airflow.providers.google_vendor.googleads.v12.resources",
    marshal="google.ads.googleads.v12",
    manifest={"Audience",},
)


class Audience(proto.Message):
    r"""Audience is an effective targeting option that lets you
    intersect different segment attributes, such as detailed
    demographics and affinities, to create audiences that represent
    sections of your target segments.

    Attributes:
        resource_name (str):
            Immutable. The resource name of the audience. Audience names
            have the form:

            ``customers/{customer_id}/audiences/{audience_id}``
        id (int):
            Output only. ID of the audience.
        status (google.ads.googleads.v12.enums.types.AudienceStatusEnum.AudienceStatus):
            Output only. Status of this audience.
            Indicates whether the audience is enabled or
            removed.
        name (str):
            Required. Name of the audience. It should be
            unique across all audiences. It must have a
            minimum length of 1 and maximum length of 255.
        description (str):
            Description of this audience.
        dimensions (Sequence[google.ads.googleads.v12.common.types.AudienceDimension]):
            Positive dimensions specifying the audience
            composition.
        exclusion_dimension (google.ads.googleads.v12.common.types.AudienceExclusionDimension):
            Negative dimension specifying the audience
            composition.
    """

    resource_name = proto.Field(proto.STRING, number=1,)
    id = proto.Field(proto.INT64, number=2,)
    status = proto.Field(
        proto.ENUM,
        number=3,
        enum=audience_status.AudienceStatusEnum.AudienceStatus,
    )
    name = proto.Field(proto.STRING, number=4,)
    description = proto.Field(proto.STRING, number=5,)
    dimensions = proto.RepeatedField(
        proto.MESSAGE, number=6, message=audiences.AudienceDimension,
    )
    exclusion_dimension = proto.Field(
        proto.MESSAGE, number=7, message=audiences.AudienceExclusionDimension,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from airflow.providers.google_vendor.googleads.v12.common.types import keyword_plan_common
from airflow.providers.google_vendor.googleads.v12.resources.types import (
    keyword_plan as gagr_keyword_plan,
)
from google.protobuf import field_mask_pb2  # type: ignore
from google.rpc import status_pb2  # type: ignore


__protobuf__ = proto.module(
    package="airflow.providers.google_vendor.googleads.v12.services",
    marshal="google.ads.googleads.v12",
    manifest={
        "MutateKeywordPlansRequest",
        "KeywordPlanOperation",
        "MutateKeywordPlansResponse",
        "MutateKeywordPlansResult",
        "GenerateForecastCurveRequest",
        "GenerateForecastCurveResponse",
        "GenerateForecastTimeSeriesRequest",
        "GenerateForecastTimeSeriesResponse",
        "GenerateForecastMetricsRequest",
        "GenerateForecastMetricsResponse",
        "KeywordPlanCampaignForecast",
        "KeywordPlanAdGroupForecast",
        "KeywordPlanKeywordForecast",
        "KeywordPlanCampaignForecastCurve",
        "KeywordPlanMaxCpcBidForecastCurve",
        "KeywordPlanMaxCpcBidForecast",
        "KeywordPlanWeeklyTimeSeriesForecast",
        "KeywordPlanWeeklyForecast",
        "ForecastMetrics",
        "GenerateHistoricalMetricsRequest",
        "GenerateHistoricalMetricsResponse",
        "KeywordPlanKeywordHistoricalMetrics",
    },
)


class MutateKeywordPlansRequest(proto.Message):
    r"""Request message for
    [KeywordPlanService.MutateKeywordPlans][google.ads.googleads.v12.services.KeywordPlanService.MutateKeywordPlans].

    Attributes:
        customer_id (str):
            Required. The ID of the customer whose
            keyword plans are being modified.
        operations (Sequence[google.ads.googleads.v12.services.types.KeywordPlanOperation]):
            Required. The list of operations to perform
            on individual keyword plans.
        partial_failure (bool):
            If true, successful operations will be
            carried out and invalid operations will return
            errors. If false, all operations will be carried
            out in one transaction if and only if they are
            all valid. Default is false.
        validate_only (bool):
            If true, the request is validated but not
            executed. Only errors are returned, not results.
    """

    customer_id = proto.Field(proto.STRING, number=1,)
    operations = proto.RepeatedField(
        proto.MESSAGE, number=2, message="KeywordPlanOperation",
    )
    partial_failure = proto.Field(proto.BOOL, number=3,)
    validate_only = proto.Field(proto.BOOL, number=4,)


class KeywordPlanOperation(proto.Message):
    r"""A single operation (create, update, remove) on a keyword
    plan.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            The FieldMask that determines which resource
            fields are modified in an update.
        create (google.ads.googleads.v12.resources.types.KeywordPlan):
            Create operation: No resource name is
            expected for the new keyword plan.

            This field is a member of `oneof`_ ``operation``.
        update (google.ads.googleads.v12.resources.types.KeywordPlan):
            Update operation: The keyword plan is
            expected to have a valid resource name.

            This field is a member of `oneof`_ ``operation``.
        remove (str):
            Remove operation: A resource name for the removed keyword
            plan is expected in this format:

            ``customers/{customer_id}/keywordPlans/{keyword_plan_id}``

            This field is a member of `oneof`_ ``operation``.
    """

    update_mask = proto.Field(
        proto.MESSAGE, number=4, message=field_mask_pb2.FieldMask,
    )
    create = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="operation",
        message=gagr_keyword_plan.KeywordPlan,
    )
    update = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="operation",
        message=gagr_keyword_plan.KeywordPlan,
    )
    remove = proto.Field(proto.STRING, number=3, oneof="operation",)


class MutateKeywordPlansResponse(proto.Message):
    r"""Response message for a keyword plan mutate.

    Attributes:
        partial_failure_error (google.rpc.status_pb2.Status):
            Errors that pertain to operation failures in the partial
            failure mode. Returned only when partial_failure = true and
            all errors occur inside the operations. If any errors occur
            outside the operations (for example, auth errors), we return
            an RPC level error.
        results (Sequence[google.ads.googleads.v12.services.types.MutateKeywordPlansResult]):
            All results for the mutate.
    """

    partial_failure_error = proto.Field(
        proto.MESSAGE, number=3, message=status_pb2.Status,
    )
    results = proto.RepeatedField(
        proto.MESSAGE, number=2, message="MutateKeywordPlansResult",
    )


class MutateKeywordPlansResult(proto.Message):
    r"""The result for the keyword plan mutate.

    Attributes:
        resource_name (str):
            Returned for successful operations.
    """

    resource_name = proto.Field(proto.STRING, number=1,)


class GenerateForecastCurveRequest(proto.Message):
    r"""Request message for
    [KeywordPlanService.GenerateForecastCurve][google.ads.googleads.v12.services.KeywordPlanService.GenerateForecastCurve].

    Attributes:
        keyword_plan (str):
            Required. The resource name of the keyword
            plan to be forecasted.
    """

    keyword_plan = proto.Field(proto.STRING, number=1,)


class GenerateForecastCurveResponse(proto.Message):
    r"""Response message for
    [KeywordPlanService.GenerateForecastCurve][google.ads.googleads.v12.services.KeywordPlanService.GenerateForecastCurve].

    Attributes:
        campaign_forecast_curves (Sequence[google.ads.googleads.v12.services.types.KeywordPlanCampaignForecastCurve]):
            List of forecast curves for the keyword plan
            campaign. One maximum.
    """

    campaign_forecast_curves = proto.RepeatedField(
        proto.MESSAGE, number=1, message="KeywordPlanCampaignForecastCurve",
    )


class GenerateForecastTimeSeriesRequest(proto.Message):
    r"""Request message for
    [KeywordPlanService.GenerateForecastTimeSeries][google.ads.googleads.v12.services.KeywordPlanService.GenerateForecastTimeSeries].

    Attributes:
        keyword_plan (str):
            Required. The resource name of the keyword
            plan to be forecasted.
    """

    keyword_plan = proto.Field(proto.STRING, number=1,)


class GenerateForecastTimeSeriesResponse(proto.Message):
    r"""Response message for
    [KeywordPlanService.GenerateForecastTimeSeries][google.ads.googleads.v12.services.KeywordPlanService.GenerateForecastTimeSeries].

    Attributes:
        weekly_time_series_forecasts (Sequence[google.ads.googleads.v12.services.types.KeywordPlanWeeklyTimeSeriesForecast]):
            List of weekly time series forecasts for the
            keyword plan campaign. One maximum.
    """

    weekly_time_series_forecasts = proto.RepeatedField(
        proto.MESSAGE, number=1, message="KeywordPlanWeeklyTimeSeriesForecast",
    )


class GenerateForecastMetricsRequest(proto.Message):
    r"""Request message for
    [KeywordPlanService.GenerateForecastMetrics][google.ads.googleads.v12.services.KeywordPlanService.GenerateForecastMetrics].

    Attributes:
        keyword_plan (str):
            Required. The resource name of the keyword
            plan to be forecasted.
    """

    keyword_plan = proto.Field(proto.STRING, number=1,)


class GenerateForecastMetricsResponse(proto.Message):
    r"""Response message for
    [KeywordPlanService.GenerateForecastMetrics][google.ads.googleads.v12.services.KeywordPlanService.GenerateForecastMetrics].

    Attributes:
        campaign_forecasts (Sequence[google.ads.googleads.v12.services.types.KeywordPlanCampaignForecast]):
            List of campaign forecasts.
            One maximum.
        ad_group_forecasts (Sequence[google.ads.googleads.v12.services.types.KeywordPlanAdGroupForecast]):
            List of ad group forecasts.
        keyword_forecasts (Sequence[google.ads.googleads.v12.services.types.KeywordPlanKeywordForecast]):
            List of keyword forecasts.
    """

    campaign_forecasts = proto.RepeatedField(
        proto.MESSAGE, number=1, message="KeywordPlanCampaignForecast",
    )
    ad_group_forecasts = proto.RepeatedField(
        proto.MESSAGE, number=2, message="KeywordPlanAdGroupForecast",
    )
    keyword_forecasts = proto.RepeatedField(
        proto.MESSAGE, number=3, message="KeywordPlanKeywordForecast",
    )


class KeywordPlanCampaignForecast(proto.Message):
    r"""A campaign forecast.

    Attributes:
        keyword_plan_campaign (str):
            The resource name of the Keyword Plan campaign related to
            the forecast.

            ``customers/{customer_id}/keywordPlanCampaigns/{keyword_plan_campaign_id}``

            This field is a member of `oneof`_ ``_keyword_plan_campaign``.
        campaign_forecast (google.ads.googleads.v12.services.types.ForecastMetrics):
            The forecast for the Keyword Plan campaign.
    """

    keyword_plan_campaign = proto.Field(proto.STRING, number=3, optional=True,)
    campaign_forecast = proto.Field(
        proto.MESSAGE, number=2, message="ForecastMetrics",
    )


class KeywordPlanAdGroupForecast(proto.Message):
    r"""An ad group forecast.

    Attributes:
        keyword_plan_ad_group (str):
            The resource name of the Keyword Plan ad group related to
            the forecast.

            ``customers/{customer_id}/keywordPlanAdGroups/{keyword_plan_ad_group_id}``

            This field is a member of `oneof`_ ``_keyword_plan_ad_group``.
        ad_group_forecast (google.ads.googleads.v12.services.types.ForecastMetrics):
            The forecast for the Keyword Plan ad group.
    """

    keyword_plan_ad_group = proto.Field(proto.STRING, number=3, optional=True,)
    ad_group_forecast = proto.Field(
        proto.MESSAGE, number=2, message="ForecastMetrics",
    )


class KeywordPlanKeywordForecast(proto.Message):
    r"""A keyword forecast.

    Attributes:
        keyword_plan_ad_group_keyword (str):
            The resource name of the Keyword Plan keyword related to the
            forecast.

            ``customers/{customer_id}/keywordPlanAdGroupKeywords/{keyword_plan_ad_group_keyword_id}``

            This field is a member of `oneof`_ ``_keyword_plan_ad_group_keyword``.
        keyword_forecast (google.ads.googleads.v12.services.types.ForecastMetrics):
            The forecast for the Keyword Plan keyword.
    """

    keyword_plan_ad_group_keyword = proto.Field(
        proto.STRING, number=3, optional=True,
    )
    keyword_forecast = proto.Field(
        proto.MESSAGE, number=2, message="ForecastMetrics",
    )


class KeywordPlanCampaignForecastCurve(proto.Message):
    r"""The forecast curve for the campaign.

    Attributes:
        keyword_plan_campaign (str):
            The resource name of the Keyword Plan campaign related to
            the forecast.

            ``customers/{customer_id}/keywordPlanCampaigns/{keyword_plan_campaign_id}``

            This field is a member of `oneof`_ ``_keyword_plan_campaign``.
        max_cpc_bid_forecast_curve (google.ads.googleads.v12.services.types.KeywordPlanMaxCpcBidForecastCurve):
            The max cpc bid forecast curve for the
            campaign.
    """

    keyword_plan_campaign = proto.Field(proto.STRING, number=3, optional=True,)
    max_cpc_bid_forecast_curve = proto.Field(
        proto.MESSAGE, number=2, message="KeywordPlanMaxCpcBidForecastCurve",
    )


class KeywordPlanMaxCpcBidForecastCurve(proto.Message):
    r"""The max cpc bid forecast curve.

    Attributes:
        max_cpc_bid_forecasts (Sequence[google.ads.googleads.v12.services.types.KeywordPlanMaxCpcBidForecast]):
            The forecasts for the Keyword Plan campaign
            at different max CPC bids.
    """

    max_cpc_bid_forecasts = proto.RepeatedField(
        proto.MESSAGE, number=1, message="KeywordPlanMaxCpcBidForecast",
    )


class KeywordPlanMaxCpcBidForecast(proto.Message):
    r"""The forecast of the campaign at a specific bid.

    Attributes:
        max_cpc_bid_micros (int):
            The max cpc bid in micros.

            This field is a member of `oneof`_ ``_max_cpc_bid_micros``.
        max_cpc_bid_forecast (google.ads.googleads.v12.services.types.ForecastMetrics):
            The forecast for the Keyword Plan campaign at
            the specific bid.
    """

    max_cpc_bid_micros = proto.Field(proto.INT64, number=3, optional=True,)
    max_cpc_bid_forecast = proto.Field(
        proto.MESSAGE, number=2, message="ForecastMetrics",
    )


class KeywordPlanWeeklyTimeSeriesForecast(proto.Message):
    r"""The weekly time series forecast for the keyword plan
    campaign.

    Attributes:
        keyword_plan_campaign (str):
            The resource name of the Keyword Plan campaign related to
            the forecast.

            ``customers/{customer_id}/keywordPlanCampaigns/{keyword_plan_campaign_id}``

            This field is a member of `oneof`_ ``_keyword_plan_campaign``.
        weekly_forecasts (Sequence[google.ads.googleads.v12.services.types.KeywordPlanWeeklyForecast]):
            The forecasts for the Keyword Plan campaign
            at different max CPC bids.
    """

    keyword_plan_campaign = proto.Field(proto.STRING, number=1, optional=True,)
    weekly_forecasts = proto.RepeatedField(
        proto.MESSAGE, number=2, message="KeywordPlanWeeklyForecast",
    )


class KeywordPlanWeeklyForecast(proto.Message):
    r"""The forecast of the campaign for the week starting start_date.

    Attributes:
        start_date (str):
            The start date, in yyyy-mm-dd format. This
            date is inclusive.

            This field is a member of `oneof`_ ``_start_date``.
        forecast (google.ads.googleads.v12.services.types.ForecastMetrics):
            The forecast for the Keyword Plan campaign
            for the week.
    """

    start_date = proto.Field(proto.STRING, number=1, optional=True,)
    forecast = proto.Field(proto.MESSAGE, number=2, message="ForecastMetrics",)


class ForecastMetrics(proto.Message):
    r"""Forecast metrics.

    Attributes:
        impressions (float):
            Impressions

            This field is a member of `oneof`_ ``_impressions``.
        ctr (float):
            Ctr

            This field is a member of `oneof`_ ``_ctr``.
        average_cpc (int):
            AVG cpc

            This field is a member of `oneof`_ ``_average_cpc``.
        clicks (float):
            Clicks

            This field is a member of `oneof`_ ``_clicks``.
        cost_micros (int):
            Cost

            This field is a member of `oneof`_ ``_cost_micros``.
    """

    impressions = proto.Field(proto.DOUBLE, number=7, optional=True,)
    ctr = proto.Field(proto.DOUBLE, number=8, optional=True,)
    average_cpc = proto.Field(proto.INT64, number=9, optional=True,)
    clicks = proto.Field(proto.DOUBLE, number=10, optional=True,)
    cost_micros = proto.Field(proto.INT64, number=11, optional=True,)


class GenerateHistoricalMetricsRequest(proto.Message):
    r"""Request message for
    [KeywordPlanService.GenerateHistoricalMetrics][google.ads.googleads.v12.services.KeywordPlanService.GenerateHistoricalMetrics].

    Attributes:
        keyword_plan (str):
            Required. The resource name of the keyword
            plan of which historical metrics are requested.
        aggregate_metrics (google.ads.googleads.v12.common.types.KeywordPlanAggregateMetrics):
            The aggregate fields to include in response.
        historical_metrics_options (google.ads.googleads.v12.common.types.HistoricalMetricsOptions):
            The options for historical metrics data.
    """

    keyword_plan = proto.Field(proto.STRING, number=1,)
    aggregate_metrics = proto.Field(
        proto.MESSAGE,
        number=2,
        message=keyword_plan_common.KeywordPlanAggregateMetrics,
    )
    historical_metrics_options = proto.Field(
        proto.MESSAGE,
        number=3,
        message=keyword_plan_common.HistoricalMetricsOptions,
    )


class GenerateHistoricalMetricsResponse(proto.Message):
    r"""Response message for
    [KeywordPlanService.GenerateHistoricalMetrics][google.ads.googleads.v12.services.KeywordPlanService.GenerateHistoricalMetrics].

    Attributes:
        metrics (Sequence[google.ads.googleads.v12.services.types.KeywordPlanKeywordHistoricalMetrics]):
            List of keyword historical metrics.
        aggregate_metric_results (google.ads.googleads.v12.common.types.KeywordPlanAggregateMetricResults):
            The aggregate metrics for all the keywords in
            the keyword planner plan.
    """

    metrics = proto.RepeatedField(
        proto.MESSAGE, number=1, message="KeywordPlanKeywordHistoricalMetrics",
    )
    aggregate_metric_results = proto.Field(
        proto.MESSAGE,
        number=2,
        message=keyword_plan_common.KeywordPlanAggregateMetricResults,
    )


class KeywordPlanKeywordHistoricalMetrics(proto.Message):
    r"""A keyword historical metrics.

    Attributes:
        search_query (str):
            The text of the query associated with one or more
            ad_group_keywords in the plan.

            Note that we de-dupe your keywords list, eliminating close
            variants before returning the plan's keywords as text. For
            example, if your plan originally contained the keywords
            'car' and 'cars', the returned search query will only
            contain 'cars'. Starting V5, the list of de-duped queries
            will be included in close_variants field.

            This field is a member of `oneof`_ ``_search_query``.
        close_variants (Sequence[str]):
            The list of close variant queries for search_query whose
            search results are combined into the search_query.
        keyword_metrics (google.ads.googleads.v12.common.types.KeywordPlanHistoricalMetrics):
            The historical metrics for the query associated with one or
            more ad_group_keywords in the plan.
    """

    search_query = proto.Field(proto.STRING, number=4, optional=True,)
    close_variants = proto.RepeatedField(proto.STRING, number=3,)
    keyword_metrics = proto.Field(
        proto.MESSAGE,
        number=2,
        message=keyword_plan_common.KeywordPlanHistoricalMetrics,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from airflow.providers.google_vendor.googleads.v12.resources.types import payments_account


__protobuf__ = proto.module(
    package="airflow.providers.google_vendor.googleads.v12.services",
    marshal="google.ads.googleads.v12",
    manifest={"ListPaymentsAccountsRequest", "ListPaymentsAccountsResponse",},
)


class ListPaymentsAccountsRequest(proto.Message):
    r"""Request message for fetching all accessible payments
    accounts.

    Attributes:
        customer_id (str):
            Required. The ID of the customer to apply the
            PaymentsAccount list operation to.
    """

    customer_id = proto.Field(proto.STRING, number=1,)


class ListPaymentsAccountsResponse(proto.Message):
    r"""Response message for
    [PaymentsAccountService.ListPaymentsAccounts][google.ads.googleads.v12.services.PaymentsAccountService.ListPaymentsAccounts].

    Attributes:
        payments_accounts (Sequence[google.ads.googleads.v12.resources.types.PaymentsAccount]):
            The list of accessible payments accounts.
    """

    payments_accounts = proto.RepeatedField(
        proto.MESSAGE, number=1, message=payments_account.PaymentsAccount,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from typing import TYPE_CHECKING

from airflow.providers.google.cloud.links.base import BaseGoogleLink

if TYPE_CHECKING:
    from airflow.utils.context import Context

BASE_LINK = "https://console.cloud.google.com/lifesciences"
LIFESCIENCES_LIST_LINK = BASE_LINK + "/pipelines?project={project_id}"


class LifeSciencesLink(BaseGoogleLink):
    """Helper class for constructing Life Sciences List link"""

    name = "Life Sciences"
    key = "lifesciences_key"
    format_str = LIFESCIENCES_LIST_LINK

    @staticmethod
    def persist(
        context: Context,
        task_instance,
        project_id: str,
    ):
        task_instance.xcom_push(
            context=context,
            key=LifeSciencesLink.key,
            value={
                "project_id": project_id,
            },
        )



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore


__protobuf__ = proto.module(
    package="airflow.providers.google_vendor.googleads.v12.enums",
    marshal="google.ads.googleads.v12",
    manifest={"AdGroupTypeEnum",},
)


class AdGroupTypeEnum(proto.Message):
    r"""Defines types of an ad group, specific to a particular
    campaign channel type. This type drives validations that
    restrict which entities can be added to the ad group.

    """

    class AdGroupType(proto.Enum):
        r"""Enum listing the possible types of an ad group."""
        UNSPECIFIED = 0
        UNKNOWN = 1
        SEARCH_STANDARD = 2
        DISPLAY_STANDARD = 3
        SHOPPING_PRODUCT_ADS = 4
        HOTEL_ADS = 6
        SHOPPING_SMART_ADS = 7
        VIDEO_BUMPER = 8
        VIDEO_TRUE_VIEW_IN_STREAM = 9
        VIDEO_TRUE_VIEW_IN_DISPLAY = 10
        VIDEO_NON_SKIPPABLE_IN_STREAM = 11
        VIDEO_OUTSTREAM = 12
        SEARCH_DYNAMIC_ADS = 13
        SHOPPING_COMPARISON_LISTING_ADS = 14
        PROMOTED_HOTEL_ADS = 15
        VIDEO_RESPONSIVE = 16
        VIDEO_EFFICIENT_REACH = 17
        SMART_CAMPAIGN_ADS = 18


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore


__protobuf__ = proto.module(
    package="airflow.providers.google_vendor.googleads.v12.errors",
    marshal="google.ads.googleads.v12",
    manifest={"KeywordPlanErrorEnum",},
)


class KeywordPlanErrorEnum(proto.Message):
    r"""Container for enum describing possible errors from applying a
    keyword plan resource (keyword plan, keyword plan campaign,
    keyword plan ad group or keyword plan keyword) or
    KeywordPlanService RPC.

    """

    class KeywordPlanError(proto.Enum):
        r"""Enum describing possible errors from applying a keyword plan."""
        UNSPECIFIED = 0
        UNKNOWN = 1
        BID_MULTIPLIER_OUT_OF_RANGE = 2
        BID_TOO_HIGH = 3
        BID_TOO_LOW = 4
        BID_TOO_MANY_FRACTIONAL_DIGITS = 5
        DAILY_BUDGET_TOO_LOW = 6
        DAILY_BUDGET_TOO_MANY_FRACTIONAL_DIGITS = 7
        INVALID_VALUE = 8
        KEYWORD_PLAN_HAS_NO_KEYWORDS = 9
        KEYWORD_PLAN_NOT_ENABLED = 10
        KEYWORD_PLAN_NOT_FOUND = 11
        MISSING_BID = 13
        MISSING_FORECAST_PERIOD = 14
        INVALID_FORECAST_DATE_RANGE = 15
        INVALID_NAME = 16


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from airflow.providers.google_vendor.googleads.v12.enums.types import asset_field_type
from airflow.providers.google_vendor.googleads.v12.enums.types import asset_link_status
from airflow.providers.google_vendor.googleads.v12.enums.types import asset_source


__protobuf__ = proto.module(
    package="airflow.providers.google_vendor.googleads.v12.resources",
    marshal="google.ads.googleads.v12",
    manifest={"CampaignAsset",},
)


class CampaignAsset(proto.Message):
    r"""A link between a Campaign and an Asset.

    Attributes:
        resource_name (str):
            Immutable. The resource name of the campaign asset.
            CampaignAsset resource names have the form:

            ``customers/{customer_id}/campaignAssets/{campaign_id}~{asset_id}~{field_type}``
        campaign (str):
            Immutable. The campaign to which the asset is
            linked.

            This field is a member of `oneof`_ ``_campaign``.
        asset (str):
            Immutable. The asset which is linked to the
            campaign.

            This field is a member of `oneof`_ ``_asset``.
        field_type (google.ads.googleads.v12.enums.types.AssetFieldTypeEnum.AssetFieldType):
            Immutable. Role that the asset takes under
            the linked campaign. Required.
        source (google.ads.googleads.v12.enums.types.AssetSourceEnum.AssetSource):
            Output only. Source of the campaign asset
            link.
        status (google.ads.googleads.v12.enums.types.AssetLinkStatusEnum.AssetLinkStatus):
            Status of the campaign asset.
    """

    resource_name = proto.Field(proto.STRING, number=1,)
    campaign = proto.Field(proto.STRING, number=6, optional=True,)
    asset = proto.Field(proto.STRING, number=7, optional=True,)
    field_type = proto.Field(
        proto.ENUM,
        number=4,
        enum=asset_field_type.AssetFieldTypeEnum.AssetFieldType,
    )
    source = proto.Field(
        proto.ENUM, number=8, enum=asset_source.AssetSourceEnum.AssetSource,
    )
    status = proto.Field(
        proto.ENUM,
        number=5,
        enum=asset_link_status.AssetLinkStatusEnum.AssetLinkStatus,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from airflow.providers.google_vendor.googleads.v12.enums.types import ad_customizer_placeholder_field
from airflow.providers.google_vendor.googleads.v12.enums.types import (
    affiliate_location_placeholder_field,
)
from airflow.providers.google_vendor.googleads.v12.enums.types import app_placeholder_field
from airflow.providers.google_vendor.googleads.v12.enums.types import call_placeholder_field
from airflow.providers.google_vendor.googleads.v12.enums.types import callout_placeholder_field
from airflow.providers.google_vendor.googleads.v12.enums.types import custom_placeholder_field
from airflow.providers.google_vendor.googleads.v12.enums.types import dsa_page_feed_criterion_field
from airflow.providers.google_vendor.googleads.v12.enums.types import education_placeholder_field
from airflow.providers.google_vendor.googleads.v12.enums.types import feed_mapping_criterion_type
from airflow.providers.google_vendor.googleads.v12.enums.types import feed_mapping_status
from airflow.providers.google_vendor.googleads.v12.enums.types import flight_placeholder_field
from airflow.providers.google_vendor.googleads.v12.enums.types import hotel_placeholder_field
from airflow.providers.google_vendor.googleads.v12.enums.types import image_placeholder_field
from airflow.providers.google_vendor.googleads.v12.enums.types import job_placeholder_field
from airflow.providers.google_vendor.googleads.v12.enums.types import local_placeholder_field
from airflow.providers.google_vendor.googleads.v12.enums.types import (
    location_extension_targeting_criterion_field,
)
from airflow.providers.google_vendor.googleads.v12.enums.types import location_placeholder_field
from airflow.providers.google_vendor.googleads.v12.enums.types import message_placeholder_field
from airflow.providers.google_vendor.googleads.v12.enums.types import (
    placeholder_type as gage_placeholder_type,
)
from airflow.providers.google_vendor.googleads.v12.enums.types import price_placeholder_field
from airflow.providers.google_vendor.googleads.v12.enums.types import promotion_placeholder_field
from airflow.providers.google_vendor.googleads.v12.enums.types import real_estate_placeholder_field
from airflow.providers.google_vendor.googleads.v12.enums.types import sitelink_placeholder_field
from airflow.providers.google_vendor.googleads.v12.enums.types import (
    structured_snippet_placeholder_field,
)
from airflow.providers.google_vendor.googleads.v12.enums.types import travel_placeholder_field


__protobuf__ = proto.module(
    package="airflow.providers.google_vendor.googleads.v12.resources",
    marshal="google.ads.googleads.v12",
    manifest={"FeedMapping", "AttributeFieldMapping",},
)


class FeedMapping(proto.Message):
    r"""A feed mapping.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. The resource name of the feed mapping. Feed
            mapping resource names have the form:

            ``customers/{customer_id}/feedMappings/{feed_id}~{feed_mapping_id}``
        feed (str):
            Immutable. The feed of this feed mapping.

            This field is a member of `oneof`_ ``_feed``.
        attribute_field_mappings (Sequence[google.ads.googleads.v12.resources.types.AttributeFieldMapping]):
            Immutable. Feed attributes to field mappings.
            These mappings are a one-to-many relationship
            meaning that 1 feed attribute can be used to
            populate multiple placeholder fields, but 1
            placeholder field can only draw data from 1 feed
            attribute. Ad Customizer is an exception, 1
            placeholder field can be mapped to multiple feed
            attributes. Required.
        status (google.ads.googleads.v12.enums.types.FeedMappingStatusEnum.FeedMappingStatus):
            Output only. Status of the feed mapping.
            This field is read-only.
        placeholder_type (google.ads.googleads.v12.enums.types.PlaceholderTypeEnum.PlaceholderType):
            Immutable. The placeholder type of this
            mapping (for example, if the mapping maps feed
            attributes to placeholder fields).

            This field is a member of `oneof`_ ``target``.
        criterion_type (google.ads.googleads.v12.enums.types.FeedMappingCriterionTypeEnum.FeedMappingCriterionType):
            Immutable. The criterion type of this mapping
            (for example, if the mapping maps feed
            attributes to criterion fields).

            This field is a member of `oneof`_ ``target``.
    """

    resource_name = proto.Field(proto.STRING, number=1,)
    feed = proto.Field(proto.STRING, number=7, optional=True,)
    attribute_field_mappings = proto.RepeatedField(
        proto.MESSAGE, number=5, message="AttributeFieldMapping",
    )
    status = proto.Field(
        proto.ENUM,
        number=6,
        enum=feed_mapping_status.FeedMappingStatusEnum.FeedMappingStatus,
    )
    placeholder_type = proto.Field(
        proto.ENUM,
        number=3,
        oneof="target",
        enum=gage_placeholder_type.PlaceholderTypeEnum.PlaceholderType,
    )
    criterion_type = proto.Field(
        proto.ENUM,
        number=4,
        oneof="target",
        enum=feed_mapping_criterion_type.FeedMappingCriterionTypeEnum.FeedMappingCriterionType,
    )


class AttributeFieldMapping(proto.Message):
    r"""Maps from feed attribute id to a placeholder or criterion
    field id.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        feed_attribute_id (int):
            Immutable. Feed attribute from which to map.

            This field is a member of `oneof`_ ``_feed_attribute_id``.
        field_id (int):
            Output only. The placeholder field ID. If a
            placeholder field enum is not published in the
            current API version, then this field will be
            populated and the field oneof will be empty.
            This field is read-only.

            This field is a member of `oneof`_ ``_field_id``.
        sitelink_field (google.ads.googleads.v12.enums.types.SitelinkPlaceholderFieldEnum.SitelinkPlaceholderField):
            Immutable. Sitelink Placeholder Fields.

            This field is a member of `oneof`_ ``field``.
        call_field (google.ads.googleads.v12.enums.types.CallPlaceholderFieldEnum.CallPlaceholderField):
            Immutable. Call Placeholder Fields.

            This field is a member of `oneof`_ ``field``.
        app_field (google.ads.googleads.v12.enums.types.AppPlaceholderFieldEnum.AppPlaceholderField):
            Immutable. App Placeholder Fields.

            This field is a member of `oneof`_ ``field``.
        location_field (google.ads.googleads.v12.enums.types.LocationPlaceholderFieldEnum.LocationPlaceholderField):
            Output only. Location Placeholder Fields.
            This field is read-only.

            This field is a member of `oneof`_ ``field``.
        affiliate_location_field (google.ads.googleads.v12.enums.types.AffiliateLocationPlaceholderFieldEnum.AffiliateLocationPlaceholderField):
            Output only. Affiliate Location Placeholder
            Fields. This field is read-only.

            This field is a member of `oneof`_ ``field``.
        callout_field (google.ads.googleads.v12.enums.types.CalloutPlaceholderFieldEnum.CalloutPlaceholderField):
            Immutable. Callout Placeholder Fields.

            This field is a member of `oneof`_ ``field``.
        structured_snippet_field (google.ads.googleads.v12.enums.types.StructuredSnippetPlaceholderFieldEnum.StructuredSnippetPlaceholderField):
            Immutable. Structured Snippet Placeholder
            Fields.

            This field is a member of `oneof`_ ``field``.
        message_field (google.ads.googleads.v12.enums.types.MessagePlaceholderFieldEnum.MessagePlaceholderField):
            Immutable. Message Placeholder Fields.

            This field is a member of `oneof`_ ``field``.
        price_field (google.ads.googleads.v12.enums.types.PricePlaceholderFieldEnum.PricePlaceholderField):
            Immutable. Price Placeholder Fields.

            This field is a member of `oneof`_ ``field``.
        promotion_field (google.ads.googleads.v12.enums.types.PromotionPlaceholderFieldEnum.PromotionPlaceholderField):
            Immutable. Promotion Placeholder Fields.

            This field is a member of `oneof`_ ``field``.
        ad_customizer_field (google.ads.googleads.v12.enums.types.AdCustomizerPlaceholderFieldEnum.AdCustomizerPlaceholderField):
            Immutable. Ad Customizer Placeholder Fields

            This field is a member of `oneof`_ ``field``.
        dsa_page_feed_field (google.ads.googleads.v12.enums.types.DsaPageFeedCriterionFieldEnum.DsaPageFeedCriterionField):
            Immutable. Dynamic Search Ad Page Feed
            Fields.

            This field is a member of `oneof`_ ``field``.
        location_extension_targeting_field (google.ads.googleads.v12.enums.types.LocationExtensionTargetingCriterionFieldEnum.LocationExtensionTargetingCriterionField):
            Immutable. Location Target Fields.

            This field is a member of `oneof`_ ``field``.
        education_field (google.ads.googleads.v12.enums.types.EducationPlaceholderFieldEnum.EducationPlaceholderField):
            Immutable. Education Placeholder Fields

            This field is a member of `oneof`_ ``field``.
        flight_field (google.ads.googleads.v12.enums.types.FlightPlaceholderFieldEnum.FlightPlaceholderField):
            Immutable. Flight Placeholder Fields

            This field is a member of `oneof`_ ``field``.
        custom_field (google.ads.googleads.v12.enums.types.CustomPlaceholderFieldEnum.CustomPlaceholderField):
            Immutable. Custom Placeholder Fields

            This field is a member of `oneof`_ ``field``.
        hotel_field (google.ads.googleads.v12.enums.types.HotelPlaceholderFieldEnum.HotelPlaceholderField):
            Immutable. Hotel Placeholder Fields

            This field is a member of `oneof`_ ``field``.
        real_estate_field (google.ads.googleads.v12.enums.types.RealEstatePlaceholderFieldEnum.RealEstatePlaceholderField):
            Immutable. Real Estate Placeholder Fields

            This field is a member of `oneof`_ ``field``.
        travel_field (google.ads.googleads.v12.enums.types.TravelPlaceholderFieldEnum.TravelPlaceholderField):
            Immutable. Travel Placeholder Fields

            This field is a member of `oneof`_ ``field``.
        local_field (google.ads.googleads.v12.enums.types.LocalPlaceholderFieldEnum.LocalPlaceholderField):
            Immutable. Local Placeholder Fields

            This field is a member of `oneof`_ ``field``.
        job_field (google.ads.googleads.v12.enums.types.JobPlaceholderFieldEnum.JobPlaceholderField):
            Immutable. Job Placeholder Fields

            This field is a member of `oneof`_ ``field``.
        image_field (google.ads.googleads.v12.enums.types.ImagePlaceholderFieldEnum.ImagePlaceholderField):
            Immutable. Image Placeholder Fields

            This field is a member of `oneof`_ ``field``.
    """

    feed_attribute_id = proto.Field(proto.INT64, number=24, optional=True,)
    field_id = proto.Field(proto.INT64, number=25, optional=True,)
    sitelink_field = proto.Field(
        proto.ENUM,
        number=3,
        oneof="field",
        enum=sitelink_placeholder_field.SitelinkPlaceholderFieldEnum.SitelinkPlaceholderField,
    )
    call_field = proto.Field(
        proto.ENUM,
        number=4,
        oneof="field",
        enum=call_placeholder_field.CallPlaceholderFieldEnum.CallPlaceholderField,
    )
    app_field = proto.Field(
        proto.ENUM,
        number=5,
        oneof="field",
        enum=app_placeholder_field.AppPlaceholderFieldEnum.AppPlaceholderField,
    )
    location_field = proto.Field(
        proto.ENUM,
        number=6,
        oneof="field",
        enum=location_placeholder_field.LocationPlaceholderFieldEnum.LocationPlaceholderField,
    )
    affiliate_location_field = proto.Field(
        proto.ENUM,
        number=7,
        oneof="field",
        enum=affiliate_location_placeholder_field.AffiliateLocationPlaceholderFieldEnum.AffiliateLocationPlaceholderField,
    )
    callout_field = proto.Field(
        proto.ENUM,
        number=8,
        oneof="field",
        enum=callout_placeholder_field.CalloutPlaceholderFieldEnum.CalloutPlaceholderField,
    )
    structured_snippet_field = proto.Field(
        proto.ENUM,
        number=9,
        oneof="field",
        enum=structured_snippet_placeholder_field.StructuredSnippetPlaceholderFieldEnum.StructuredSnippetPlaceholderField,
    )
    message_field = proto.Field(
        proto.ENUM,
        number=10,
        oneof="field",
        enum=message_placeholder_field.MessagePlaceholderFieldEnum.MessagePlaceholderField,
    )
    price_field = proto.Field(
        proto.ENUM,
        number=11,
        oneof="field",
        enum=price_placeholder_field.PricePlaceholderFieldEnum.PricePlaceholderField,
    )
    promotion_field = proto.Field(
        proto.ENUM,
        number=12,
        oneof="field",
        enum=promotion_placeholder_field.PromotionPlaceholderFieldEnum.PromotionPlaceholderField,
    )
    ad_customizer_field = proto.Field(
        proto.ENUM,
        number=13,
        oneof="field",
        enum=ad_customizer_placeholder_field.AdCustomizerPlaceholderFieldEnum.AdCustomizerPlaceholderField,
    )
    dsa_page_feed_field = proto.Field(
        proto.ENUM,
        number=14,
        oneof="field",
        enum=dsa_page_feed_criterion_field.DsaPageFeedCriterionFieldEnum.DsaPageFeedCriterionField,
    )
    location_extension_targeting_field = proto.Field(
        proto.ENUM,
        number=15,
        oneof="field",
        enum=location_extension_targeting_criterion_field.LocationExtensionTargetingCriterionFieldEnum.LocationExtensionTargetingCriterionField,
    )
    education_field = proto.Field(
        proto.ENUM,
        number=16,
        oneof="field",
        enum=education_placeholder_field.EducationPlaceholderFieldEnum.EducationPlaceholderField,
    )
    flight_field = proto.Field(
        proto.ENUM,
        number=17,
        oneof="field",
        enum=flight_placeholder_field.FlightPlaceholderFieldEnum.FlightPlaceholderField,
    )
    custom_field = proto.Field(
        proto.ENUM,
        number=18,
        oneof="field",
        enum=custom_placeholder_field.CustomPlaceholderFieldEnum.CustomPlaceholderField,
    )
    hotel_field = proto.Field(
        proto.ENUM,
        number=19,
        oneof="field",
        enum=hotel_placeholder_field.HotelPlaceholderFieldEnum.HotelPlaceholderField,
    )
    real_estate_field = proto.Field(
        proto.ENUM,
        number=20,
        oneof="field",
        enum=real_estate_placeholder_field.RealEstatePlaceholderFieldEnum.RealEstatePlaceholderField,
    )
    travel_field = proto.Field(
        proto.ENUM,
        number=21,
        oneof="field",
        enum=travel_placeholder_field.TravelPlaceholderFieldEnum.TravelPlaceholderField,
    )
    local_field = proto.Field(
        proto.ENUM,
        number=22,
        oneof="field",
        enum=local_placeholder_field.LocalPlaceholderFieldEnum.LocalPlaceholderField,
    )
    job_field = proto.Field(
        proto.ENUM,
        number=23,
        oneof="field",
        enum=job_placeholder_field.JobPlaceholderFieldEnum.JobPlaceholderField,
    )
    image_field = proto.Field(
        proto.ENUM,
        number=26,
        oneof="field",
        enum=image_placeholder_field.ImagePlaceholderFieldEnum.ImagePlaceholderField,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
"""
This module contains a Google Kubernetes Engine Hook.

.. spelling::

    gapic
    enums
"""
from __future__ import annotations

import contextlib
import json
import time
import warnings
from typing import Sequence

import google.auth.credentials
from gcloud.aio.auth import Token
from google.api_core.exceptions import NotFound
from google.api_core.gapic_v1.method import DEFAULT, _MethodDefault
from google.api_core.retry import Retry
from google.auth.transport import requests as google_requests

# not sure why but mypy complains on missing `container_v1` but it is clearly there and is importable
from google.cloud import container_v1, exceptions  # type: ignore[attr-defined]
from google.cloud.container_v1 import ClusterManagerAsyncClient, ClusterManagerClient
from google.cloud.container_v1.types import Cluster, Operation
from kubernetes import client
from kubernetes_asyncio import client as async_client
from kubernetes_asyncio.client.models import V1Pod
from kubernetes_asyncio.config.kube_config import FileOrData
from urllib3.exceptions import HTTPError

from airflow import version
from airflow.compat.functools import cached_property
from airflow.exceptions import AirflowException
from airflow.kubernetes.pod_generator_deprecated import PodDefaults
from airflow.providers.google.common.consts import CLIENT_INFO
from airflow.providers.google.common.hooks.base_google import (
    PROVIDE_PROJECT_ID,
    GoogleBaseAsyncHook,
    GoogleBaseHook,
)

OPERATIONAL_POLL_INTERVAL = 15


class GKEHook(GoogleBaseHook):
    """
    Hook for managing Google Kubernetes Engine cluster APIs.

    All the methods in the hook where project_id is used must be called with
    keyword arguments rather than positional.
    """

    def __init__(
        self,
        gcp_conn_id: str = "google_cloud_default",
        delegate_to: str | None = None,
        location: str | None = None,
        impersonation_chain: str | Sequence[str] | None = None,
    ) -> None:
        super().__init__(
            gcp_conn_id=gcp_conn_id,
            delegate_to=delegate_to,
            impersonation_chain=impersonation_chain,
        )
        self._client: ClusterManagerClient | None = None
        self.location = location

    def get_cluster_manager_client(self) -> ClusterManagerClient:
        """Returns ClusterManagerClient."""
        if self._client is None:
            self._client = ClusterManagerClient(credentials=self.get_credentials(), client_info=CLIENT_INFO)
        return self._client

    # To preserve backward compatibility
    # TODO: remove one day
    def get_conn(self) -> container_v1.ClusterManagerClient:
        warnings.warn(
            "The get_conn method has been deprecated. You should use the get_cluster_manager_client method.",
            DeprecationWarning,
        )
        return self.get_cluster_manager_client()

    # To preserve backward compatibility
    # TODO: remove one day
    def get_client(self) -> ClusterManagerClient:
        warnings.warn(
            "The get_client method has been deprecated. You should use the get_conn method.",
            DeprecationWarning,
        )
        return self.get_conn()

    def wait_for_operation(self, operation: Operation, project_id: str | None = None) -> Operation:
        """
        Given an operation, continuously fetches the status from Google Cloud until either
        completion or an error occurring

        :param operation: The Operation to wait for
        :param project_id: Google Cloud project ID
        :return: A new, updated operation fetched from Google Cloud
        """
        self.log.info("Waiting for OPERATION_NAME %s", operation.name)
        time.sleep(OPERATIONAL_POLL_INTERVAL)
        while operation.status != Operation.Status.DONE:
            if operation.status == Operation.Status.RUNNING or operation.status == Operation.Status.PENDING:
                time.sleep(OPERATIONAL_POLL_INTERVAL)
            else:
                raise exceptions.GoogleCloudError(f"Operation has failed with status: {operation.status}")
            # To update status of operation
            operation = self.get_operation(operation.name, project_id=project_id or self.project_id)
        return operation

    def get_operation(self, operation_name: str, project_id: str | None = None) -> Operation:
        """
        Fetches the operation from Google Cloud

        :param operation_name: Name of operation to fetch
        :param project_id: Google Cloud project ID
        :return: The new, updated operation from Google Cloud
        """
        return self.get_cluster_manager_client().get_operation(
            name=(
                f"projects/{project_id or self.project_id}"
                f"/locations/{self.location}/operations/{operation_name}"
            )
        )

    @staticmethod
    def _append_label(cluster_proto: Cluster, key: str, val: str) -> Cluster:
        """
        Append labels to provided Cluster Protobuf

        Labels must fit the regex ``[a-z]([-a-z0-9]*[a-z0-9])?`` (current
         airflow version string follows semantic versioning spec: x.y.z).

        :param cluster_proto: The proto to append resource_label airflow
            version to
        :param key: The key label
        :param val:
        :return: The cluster proto updated with new label
        """
        val = val.replace(".", "-").replace("+", "-")
        cluster_proto.resource_labels.update({key: val})
        return cluster_proto

    @GoogleBaseHook.fallback_to_default_project_id
    def delete_cluster(
        self,
        name: str,
        project_id: str = PROVIDE_PROJECT_ID,
        wait_to_complete: bool = True,
        retry: Retry | _MethodDefault = DEFAULT,
        timeout: float | None = None,
    ) -> Operation | None:
        """
        Deletes the cluster, including the Kubernetes endpoint and all
        worker nodes. Firewalls and routes that were configured during
        cluster creation are also deleted. Other Google Compute Engine
        resources that might be in use by the cluster (e.g. load balancer
        resources) will not be deleted if they were not present at the
        initial create time.

        :param name: The name of the cluster to delete
        :param project_id: Google Cloud project ID
        :param wait_to_complete: A boolean value which makes method to sleep while
            operation of deletion is not finished.
        :param retry: Retry object used to determine when/if to retry requests.
            If None is specified, requests will not be retried.
        :param timeout: The amount of time, in seconds, to wait for the request to
            complete. Note that if retry is specified, the timeout applies to each
            individual attempt.
        :return: The full url to the delete operation if successful, else None
        """
        self.log.info("Deleting (project_id=%s, location=%s, cluster_id=%s)", project_id, self.location, name)

        try:
            operation = self.get_cluster_manager_client().delete_cluster(
                name=f"projects/{project_id}/locations/{self.location}/clusters/{name}",
                retry=retry,
                timeout=timeout,
            )
            if wait_to_complete:
                operation = self.wait_for_operation(operation, project_id)
            # Returns server-defined url for the resource
            return operation
        except NotFound as error:
            self.log.info("Assuming Success: %s", error.message)
            return None

    @GoogleBaseHook.fallback_to_default_project_id
    def create_cluster(
        self,
        cluster: dict | Cluster,
        project_id: str = PROVIDE_PROJECT_ID,
        wait_to_complete: bool = True,
        retry: Retry | _MethodDefault = DEFAULT,
        timeout: float | None = None,
    ) -> Operation | Cluster:
        """
        Creates a cluster, consisting of the specified number and type of Google Compute
        Engine instances.

        :param cluster: A Cluster protobuf or dict. If dict is provided, it must
            be of the same form as the protobuf message
            :class:`google.cloud.container_v1.types.Cluster`
        :param project_id: Google Cloud project ID
        :param wait_to_complete: A boolean value which makes method to sleep while
            operation of creation is not finished.
        :param retry: A retry object (``google.api_core.retry.Retry``) used to
            retry requests.
            If None is specified, requests will not be retried.
        :param timeout: The amount of time, in seconds, to wait for the request to
            complete. Note that if retry is specified, the timeout applies to each
            individual attempt.
        :return: The full url to the new, or existing, cluster
        :raises:
            ParseError: On JSON parsing problems when trying to convert dict
            AirflowException: cluster is not dict type nor Cluster proto type
        """
        if isinstance(cluster, dict):
            cluster = Cluster.from_json(json.dumps(cluster))
        elif not isinstance(cluster, Cluster):
            raise AirflowException("cluster is not instance of Cluster proto or python dict")

        self._append_label(cluster, "airflow-version", "v" + version.version)  # type: ignore

        self.log.info(
            "Creating (project_id=%s, location=%s, cluster_name=%s)",
            project_id,
            self.location,
            cluster.name,  # type: ignore
        )
        operation = self.get_cluster_manager_client().create_cluster(
            parent=f"projects/{project_id}/locations/{self.location}",
            cluster=cluster,  # type: ignore
            retry=retry,
            timeout=timeout,
        )

        if wait_to_complete:
            operation = self.wait_for_operation(operation, project_id)

        return operation

    @GoogleBaseHook.fallback_to_default_project_id
    def get_cluster(
        self,
        name: str,
        project_id: str = PROVIDE_PROJECT_ID,
        retry: Retry | _MethodDefault = DEFAULT,
        timeout: float | None = None,
    ) -> Cluster:
        """
        Gets details of specified cluster

        :param name: The name of the cluster to retrieve
        :param project_id: Google Cloud project ID
        :param retry: A retry object used to retry requests. If None is specified,
            requests will not be retried.
        :param timeout: The amount of time, in seconds, to wait for the request to
            complete. Note that if retry is specified, the timeout applies to each
            individual attempt.
        :return: google.cloud.container_v1.types.Cluster
        """
        self.log.info(
            "Fetching cluster (project_id=%s, location=%s, cluster_name=%s)",
            project_id or self.project_id,
            self.location,
            name,
        )

        return self.get_cluster_manager_client().get_cluster(
            name=f"projects/{project_id}/locations/{self.location}/clusters/{name}",
            retry=retry,
            timeout=timeout,
        )


class GKEAsyncHook(GoogleBaseAsyncHook):
    """Hook implemented with usage of asynchronous client of GKE."""

    sync_hook_class = GKEHook

    def __init__(
        self,
        gcp_conn_id: str = "google_cloud_default",
        delegate_to: str | None = None,
        location: str | None = None,
        impersonation_chain: str | Sequence[str] | None = None,
    ) -> None:
        super().__init__(
            gcp_conn_id=gcp_conn_id,
            delegate_to=delegate_to,
            impersonation_chain=impersonation_chain,
        )
        self._client: ClusterManagerAsyncClient | None = None
        self.location = location

    async def _get_client(self) -> ClusterManagerAsyncClient:
        if self._client is None:
            self._client = ClusterManagerAsyncClient(
                credentials=(await self.get_sync_hook()).get_credentials(),
                client_info=CLIENT_INFO,
            )
        return self._client

    @GoogleBaseHook.fallback_to_default_project_id
    async def get_operation(
        self,
        operation_name: str,
        project_id: str = PROVIDE_PROJECT_ID,
    ) -> Operation:
        """
        Fetches the operation from Google Cloud.

        :param operation_name: Name of operation to fetch.
        :param project_id: Google Cloud project ID.
        :return: The new, updated operation from Google Cloud.
        """
        project_id = project_id or (await self.get_sync_hook()).project_id

        operation_path = f"projects/{project_id}/locations/{self.location}/operations/{operation_name}"
        client = await self._get_client()
        return await client.get_operation(
            name=operation_path,
        )


class GKEPodHook(GoogleBaseHook):
    """Hook for managing Google Kubernetes Engine pod APIs."""

    def __init__(
        self,
        cluster_url: str,
        ssl_ca_cert: str,
        *args,
        **kwargs,
    ):
        super().__init__(*args, **kwargs)
        self._cluster_url = cluster_url
        self._ssl_ca_cert = ssl_ca_cert

    @cached_property
    def api_client(self) -> client.ApiClient:
        return self.get_conn()

    @cached_property
    def core_v1_client(self) -> client.CoreV1Api:
        return client.CoreV1Api(self.api_client)

    @property
    def is_in_cluster(self) -> bool:
        return False

    @staticmethod
    def get_xcom_sidecar_container_image():
        """Returns the xcom sidecar image that defined in the connection"""
        return PodDefaults.SIDECAR_CONTAINER.image

    def get_conn(self) -> client.ApiClient:
        configuration = self._get_config()
        return client.ApiClient(configuration)

    def _get_config(self) -> client.configuration.Configuration:
        configuration = client.Configuration(
            host=self._cluster_url,
            api_key_prefix={"authorization": "Bearer"},
            api_key={"authorization": self._get_token(self.get_credentials())},
        )
        configuration.ssl_ca_cert = FileOrData(
            {
                "certificate-authority-data": self._ssl_ca_cert,
            },
            file_key_name="certificate-authority",
        ).as_file()
        return configuration

    @staticmethod
    def _get_token(creds: google.auth.credentials.Credentials) -> str:
        if creds.token is None or creds.expired:
            auth_req = google_requests.Request()
            creds.refresh(auth_req)
        return creds.token

    def get_pod(self, name: str, namespace: str) -> V1Pod:
        """
        Gets pod's object.

        :param name: Name of the pod.
        :param namespace: Name of the pod's namespace.
        """
        return self.core_v1_client.read_namespaced_pod(
            name=name,
            namespace=namespace,
        )


class GKEPodAsyncHook(GoogleBaseAsyncHook):
    """
    Hook for managing Google Kubernetes Engine pods APIs in asynchronous way.

    :param cluster_url: The URL pointed to the cluster.
    :param ssl_ca_cert: SSL certificate that is used for authentication to the pod.
    """

    sync_hook_class = GKEPodHook
    scopes = ["https://www.googleapis.com/auth/cloud-platform"]

    def __init__(
        self,
        cluster_url: str,
        ssl_ca_cert: str,
        **kwargs,
    ):

        self._cluster_url = cluster_url
        self._ssl_ca_cert = ssl_ca_cert

        kwargs.update(
            cluster_url=cluster_url,
            ssl_ca_cert=ssl_ca_cert,
        )
        super().__init__(**kwargs)

    @contextlib.asynccontextmanager
    async def get_conn(self, token: Token) -> async_client.ApiClient:  # type: ignore[override]
        kube_client = None
        try:
            kube_client = await self._load_config(token)
            yield kube_client
        finally:
            if kube_client is not None:
                await kube_client.close()

    async def _load_config(self, token: Token) -> async_client.ApiClient:
        configuration = self._get_config()
        access_token = await token.get()
        return async_client.ApiClient(
            configuration,
            header_name="Authorization",
            header_value=f"Bearer {access_token}",
        )

    def _get_config(self) -> async_client.configuration.Configuration:
        configuration = async_client.Configuration(
            host=self._cluster_url,
            ssl_ca_cert=FileOrData(
                {
                    "certificate-authority-data": self._ssl_ca_cert,
                },
                file_key_name="certificate-authority",
            ).as_file(),
        )
        return configuration

    async def get_pod(self, name: str, namespace: str) -> V1Pod:
        """
        Gets pod's object.

        :param name: Name of the pod.
        :param namespace: Name of the pod's namespace.
        """
        async with Token(scopes=self.scopes) as token:
            async with self.get_conn(token) as connection:
                v1_api = async_client.CoreV1Api(connection)
                pod: V1Pod = await v1_api.read_namespaced_pod(
                    name=name,
                    namespace=namespace,
                )
            return pod

    async def delete_pod(self, name: str, namespace: str):
        """
        Deletes pod's object.

        :param name: Name of the pod.
        :param namespace: Name of the pod's namespace.
        """
        async with Token(scopes=self.scopes) as token:
            async with self.get_conn(token) as connection:
                try:
                    v1_api = async_client.CoreV1Api(connection)
                    await v1_api.delete_namespaced_pod(
                        name=name,
                        namespace=namespace,
                        body=client.V1DeleteOptions(),
                    )
                except async_client.ApiException as e:
                    # If the pod is already deleted
                    if e.status != 404:
                        raise

    async def read_logs(self, name: str, namespace: str):
        """
        Reads logs inside the pod while starting containers inside. All the logs will be outputted with its
        timestamp to track the logs after the execution of the pod is completed. The method is used for async
        output of the logs only in the pod failed it execution or the task was cancelled by the user.

        :param name: Name of the pod.
        :param namespace: Name of the pod's namespace.
        """
        async with Token(scopes=self.scopes) as token:
            async with self.get_conn(token) as connection:
                try:
                    v1_api = async_client.CoreV1Api(connection)
                    logs = await v1_api.read_namespaced_pod_log(
                        name=name,
                        namespace=namespace,
                        follow=False,
                        timestamps=True,
                    )
                    logs = logs.splitlines()
                    for line in logs:
                        self.log.info("Container logs from %s", line)
                    return logs
                except HTTPError:
                    self.log.exception("There was an error reading the kubernetes API.")
                    raise



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""A mixin class to store shared functionality for all the gRPC Interceptors.

This mixin class centralizes sets of functionality that are common across all
Interceptors, including retrieving data from gRPC metadata and initializing
instances of grpc.ClientCallDetails.
"""

from dataclasses import dataclass
from importlib import import_module
from typing import AnyStr, Optional, Sequence, Tuple
import json

from google.protobuf.message import DecodeError
from grpc import ClientCallDetails, StatusCode, CallCredentials

from airflow.providers.google_vendor.googleads.errors import GoogleAdsException

_REQUEST_ID_KEY = "request-id"
# Codes that are retried upon by google.api_core.
_RETRY_STATUS_CODES = (StatusCode.INTERNAL, StatusCode.RESOURCE_EXHAUSTED)


class Interceptor:
    _SENSITIVE_INFO_MASK = "REDACTED"

    @dataclass
    class _ClientCallDetails(ClientCallDetails):
        """Wrapper class for initializing a new ClientCallDetails instance."""

        method: str
        timeout: Optional[float]
        metadata: Optional[Sequence[Tuple[str, AnyStr]]]
        credentials: Optional[CallCredentials]

    @classmethod
    def get_request_id_from_metadata(cls, trailing_metadata):
        """Gets the request ID for the Google Ads API request.

        Args:
            trailing_metadata: a tuple of metadatum from the service response.

        Returns:
            A str request ID associated with the Google Ads API request, or None
            if it doesn't exist.
        """
        for kv in trailing_metadata:
            if kv[0] == _REQUEST_ID_KEY:
                return kv[1]  # Return the found request ID.

        return None

    @classmethod
    def parse_metadata_to_json(cls, metadata):
        """Parses metadata from gRPC request and response messages to a JSON str.

        Obscures the value for "developer-token".

        Args:
            metadata: a tuple of metadatum.

        Returns:
            A str of metadata formatted as JSON key/value pairs.
        """
        metadata_dict = {}

        if metadata is None:
            return "{}"

        for datum in metadata:
            key = datum[0]
            if key == "developer-token":
                metadata_dict[key] = cls._SENSITIVE_INFO_MASK
            else:
                value = datum[1]
                metadata_dict[key] = value

        return cls.format_json_object(metadata_dict)

    @classmethod
    def format_json_object(cls, obj):
        """Parses a serializable object into a consistently formatted JSON string.

        Returns:
            A str of formatted JSON serialized from the given object.

        Args:
            obj: an object or dict.

        Returns:
            A str of metadata formatted as JSON key/value pairs.
        """

        def default_serializer(value):
            if isinstance(value, bytes):
                return value.decode(errors="ignore")
            else:
                return None

        return str(
            json.dumps(
                obj,
                indent=2,
                sort_keys=True,
                ensure_ascii=False,
                default=default_serializer,
                separators=(",", ": "),
            )
        )

    @classmethod
    def get_trailing_metadata_from_interceptor_exception(cls, exception):
        """Retrieves trailing metadata from an exception object.

        Args:
            exception: an instance of grpc.Call.

        Returns:
            A tuple of trailing metadata key value pairs.
        """
        try:
            # GoogleAdsFailure exceptions will contain trailing metadata on the
            # error attribute.
            return exception.error.trailing_metadata()
        except AttributeError:
            try:
                # Transport failures, i.e. issues at the gRPC layer, will contain
                # trailing metadata on the exception itself.
                return exception.trailing_metadata()
            except AttributeError:
                # if trailing metadata is not found in either location then
                # return an empty tuple
                return tuple()

    @classmethod
    def get_client_call_details_instance(
        cls, method, timeout, metadata, credentials=None
    ):
        """Initializes an instance of the ClientCallDetails with the given data.

        Args:
            method: A str of the service method being invoked.
            timeout: A float of the request timeout
            metadata: A list of metadata tuples
            credentials: An optional grpc.CallCredentials instance for the RPC

        Returns:
            An instance of _ClientCallDetails that wraps grpc.ClientCallDetails.
        """
        return cls._ClientCallDetails(method, timeout, metadata, credentials)

    def __init__(self, api_version):
        self._error_protos = None
        self._failure_key = (
            f"google.ads.googleads.{api_version}.errors.googleadsfailure-bin"
        )
        self._api_version = api_version

    def _get_error_from_response(self, response):
        """Attempts to wrap failed responses as GoogleAdsException instances.

        Handles failed gRPC responses of by attempting to convert them
        to a more readable GoogleAdsException. Certain types of exceptions are
        not converted; if the object's trailing metadata does not indicate that
        it is a GoogleAdsException, or if it falls under a certain category of
        status code, (INTERNAL or RESOURCE_EXHAUSTED). See documentation for
        more information about gRPC status codes:
        https://github.com/grpc/grpc/blob/master/doc/statuscodes.md

        Args:
            response: a grpc.Call/grpc.Future instance.

        Returns:
            GoogleAdsException: If the exception's trailing metadata
                indicates that it is a GoogleAdsException.
            RpcError: If the exception's is a gRPC exception but the trailing
                metadata is empty or is not indicative of a GoogleAdsException,
                or if the exception has a status code of INTERNAL or
                RESOURCE_EXHAUSTED.
            Exception: If not a GoogleAdsException or RpcException the error
                will be raised as-is.
        """
        status_code = response.code()
        response_exception = response.exception()

        if status_code not in _RETRY_STATUS_CODES:
            trailing_metadata = response.trailing_metadata()
            google_ads_failure = self._get_google_ads_failure(trailing_metadata)

            if google_ads_failure:
                request_id = self.get_request_id_from_metadata(
                    trailing_metadata
                )

                # If exception is a GoogleAdsFailure then it gets wrapped in a
                # library-specific Error type for easy handling. These errors
                # originate from the Google Ads API and are often caused by
                # invalid requests.
                return GoogleAdsException(
                    response_exception, response, google_ads_failure, request_id
                )
            else:
                # Raise the original exception if not a GoogleAdsFailure. This
                # type of error is generally caused by problems at the request
                # level, such as when an invalid endpoint is given.
                return response_exception
        else:
            # Raise the original exception if error has status code
            # INTERNAL or RESOURCE_EXHAUSTED, meaning that
            return response_exception

    def _get_google_ads_failure(self, trailing_metadata):
        """Gets the Google Ads failure details if they exist.

        Args:
            trailing_metadata: a tuple of metadatum from the service response.

        Returns:
            A GoogleAdsFailure that describes how a GoogleAds API call failed.
            Returns None if either the trailing metadata of the request did not
            return the failure details, or if the GoogleAdsFailure fails to
            parse.
        """
        if trailing_metadata is not None:
            for kv in trailing_metadata:
                if kv[0] == self._failure_key:
                    try:
                        if not self._error_protos:
                            self._error_protos = import_module(
                                f"google.ads.googleads.{self._api_version}.errors."
                                "types.errors"
                            )

                        return self._error_protos.GoogleAdsFailure.deserialize(
                            kv[1]
                        )
                    except DecodeError:
                        return None

        return None



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore


__protobuf__ = proto.module(
    package="airflow.providers.google_vendor.googleads.v12.resources",
    marshal="google.ads.googleads.v12",
    manifest={"CarrierConstant",},
)


class CarrierConstant(proto.Message):
    r"""A carrier criterion that can be used in campaign targeting.

    Attributes:
        resource_name (str):
            Output only. The resource name of the carrier criterion.
            Carrier criterion resource names have the form:

            ``carrierConstants/{criterion_id}``
        id (int):
            Output only. The ID of the carrier criterion.

            This field is a member of `oneof`_ ``_id``.
        name (str):
            Output only. The full name of the carrier in
            English.

            This field is a member of `oneof`_ ``_name``.
        country_code (str):
            Output only. The country code of the country
            where the carrier is located, for example, "AR",
            "FR", etc.

            This field is a member of `oneof`_ ``_country_code``.
    """

    resource_name = proto.Field(proto.STRING, number=1,)
    id = proto.Field(proto.INT64, number=5, optional=True,)
    name = proto.Field(proto.STRING, number=6, optional=True,)
    country_code = proto.Field(proto.STRING, number=7, optional=True,)


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore


__protobuf__ = proto.module(
    package="airflow.providers.google_vendor.googleads.v12.enums",
    marshal="google.ads.googleads.v12",
    manifest={"DisplayUploadProductTypeEnum",},
)


class DisplayUploadProductTypeEnum(proto.Message):
    r"""Container for display upload product types. Product types
    that have the word "DYNAMIC" in them must be associated with a
    campaign that has a dynamic remarketing feed. See
    https://support.google.com/google-ads/answer/6053288 for more
    info about dynamic remarketing. Other product types are regarded
    as "static" and do not have this requirement.

    """

    class DisplayUploadProductType(proto.Enum):
        r"""Enumerates display upload product types."""
        UNSPECIFIED = 0
        UNKNOWN = 1
        HTML5_UPLOAD_AD = 2
        DYNAMIC_HTML5_EDUCATION_AD = 3
        DYNAMIC_HTML5_FLIGHT_AD = 4
        DYNAMIC_HTML5_HOTEL_RENTAL_AD = 5
        DYNAMIC_HTML5_JOB_AD = 6
        DYNAMIC_HTML5_LOCAL_AD = 7
        DYNAMIC_HTML5_REAL_ESTATE_AD = 8
        DYNAMIC_HTML5_CUSTOM_AD = 9
        DYNAMIC_HTML5_TRAVEL_AD = 10
        DYNAMIC_HTML5_HOTEL_AD = 11


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore


__protobuf__ = proto.module(
    package="airflow.providers.google_vendor.googleads.v12.errors",
    marshal="google.ads.googleads.v12",
    manifest={"BillingSetupErrorEnum",},
)


class BillingSetupErrorEnum(proto.Message):
    r"""Container for enum describing possible billing setup errors.
    """

    class BillingSetupError(proto.Enum):
        r"""Enum describing possible billing setup errors."""
        UNSPECIFIED = 0
        UNKNOWN = 1
        CANNOT_USE_EXISTING_AND_NEW_ACCOUNT = 2
        CANNOT_REMOVE_STARTED_BILLING_SETUP = 3
        CANNOT_CHANGE_BILLING_TO_SAME_PAYMENTS_ACCOUNT = 4
        BILLING_SETUP_NOT_PERMITTED_FOR_CUSTOMER_STATUS = 5
        INVALID_PAYMENTS_ACCOUNT = 6
        BILLING_SETUP_NOT_PERMITTED_FOR_CUSTOMER_CATEGORY = 7
        INVALID_START_TIME_TYPE = 8
        THIRD_PARTY_ALREADY_HAS_BILLING = 9
        BILLING_SETUP_IN_PROGRESS = 10
        NO_SIGNUP_PERMISSION = 11
        CHANGE_OF_BILL_TO_IN_PROGRESS = 12
        PAYMENTS_PROFILE_NOT_FOUND = 13
        PAYMENTS_ACCOUNT_NOT_FOUND = 14
        PAYMENTS_PROFILE_INELIGIBLE = 15
        PAYMENTS_ACCOUNT_INELIGIBLE = 16
        CUSTOMER_NEEDS_INTERNAL_APPROVAL = 17
        PAYMENTS_ACCOUNT_INELIGIBLE_CURRENCY_CODE_MISMATCH = 19
        FUTURE_START_TIME_PROHIBITED = 20
        TOO_MANY_BILLING_SETUPS_FOR_PAYMENTS_ACCOUNT = 21


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore


__protobuf__ = proto.module(
    package="airflow.providers.google_vendor.googleads.v12.errors",
    marshal="google.ads.googleads.v12",
    manifest={"AssetLinkErrorEnum",},
)


class AssetLinkErrorEnum(proto.Message):
    r"""Container for enum describing possible asset link errors.
    """

    class AssetLinkError(proto.Enum):
        r"""Enum describing possible asset link errors."""
        UNSPECIFIED = 0
        UNKNOWN = 1
        PINNING_UNSUPPORTED = 2
        UNSUPPORTED_FIELD_TYPE = 3
        FIELD_TYPE_INCOMPATIBLE_WITH_ASSET_TYPE = 4
        FIELD_TYPE_INCOMPATIBLE_WITH_CAMPAIGN_TYPE = 5
        INCOMPATIBLE_ADVERTISING_CHANNEL_TYPE = 6
        IMAGE_NOT_WITHIN_SPECIFIED_DIMENSION_RANGE = 7
        INVALID_PINNED_FIELD = 8
        MEDIA_BUNDLE_ASSET_FILE_SIZE_TOO_LARGE = 9
        NOT_ENOUGH_AVAILABLE_ASSET_LINKS_FOR_VALID_COMBINATION = 10
        NOT_ENOUGH_AVAILABLE_ASSET_LINKS_WITH_FALLBACK = 11
        NOT_ENOUGH_AVAILABLE_ASSET_LINKS_WITH_FALLBACK_FOR_VALID_COMBINATION = (
            12
        )
        YOUTUBE_VIDEO_REMOVED = 13
        YOUTUBE_VIDEO_TOO_LONG = 14
        YOUTUBE_VIDEO_TOO_SHORT = 15
        EXCLUDED_PARENT_FIELD_TYPE = 16
        INVALID_STATUS = 17
        YOUTUBE_VIDEO_DURATION_NOT_DEFINED = 18
        CANNOT_CREATE_AUTOMATICALLY_CREATED_LINKS = 19
        CANNOT_LINK_TO_AUTOMATICALLY_CREATED_ASSET = 20
        CANNOT_MODIFY_ASSET_LINK_SOURCE = 21


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from airflow.providers.google_vendor.googleads.v12.enums.types import (
    response_content_type as gage_response_content_type,
)
from airflow.providers.google_vendor.googleads.v12.resources.types import (
    smart_campaign_setting as gagr_smart_campaign_setting,
)
from google.protobuf import field_mask_pb2  # type: ignore
from google.rpc import status_pb2  # type: ignore


__protobuf__ = proto.module(
    package="airflow.providers.google_vendor.googleads.v12.services",
    marshal="google.ads.googleads.v12",
    manifest={
        "MutateSmartCampaignSettingsRequest",
        "SmartCampaignSettingOperation",
        "MutateSmartCampaignSettingsResponse",
        "MutateSmartCampaignSettingResult",
    },
)


class MutateSmartCampaignSettingsRequest(proto.Message):
    r"""Request message for
    [SmartCampaignSettingService.MutateSmartCampaignSetting][].

    Attributes:
        customer_id (str):
            Required. The ID of the customer whose Smart
            campaign settings are being modified.
        operations (Sequence[google.ads.googleads.v12.services.types.SmartCampaignSettingOperation]):
            Required. The list of operations to perform
            on individual Smart campaign settings.
        partial_failure (bool):
            If true, successful operations will be
            carried out and invalid operations will return
            errors. If false, all operations will be carried
            out in one transaction if and only if they are
            all valid. Default is false.
        validate_only (bool):
            If true, the request is validated but not
            executed. Only errors are returned, not results.
        response_content_type (google.ads.googleads.v12.enums.types.ResponseContentTypeEnum.ResponseContentType):
            The response content type setting. Determines
            whether the mutable resource or just the
            resource name should be returned post mutation.
    """

    customer_id = proto.Field(proto.STRING, number=1,)
    operations = proto.RepeatedField(
        proto.MESSAGE, number=2, message="SmartCampaignSettingOperation",
    )
    partial_failure = proto.Field(proto.BOOL, number=3,)
    validate_only = proto.Field(proto.BOOL, number=4,)
    response_content_type = proto.Field(
        proto.ENUM,
        number=5,
        enum=gage_response_content_type.ResponseContentTypeEnum.ResponseContentType,
    )


class SmartCampaignSettingOperation(proto.Message):
    r"""A single operation to update Smart campaign settings for a
    campaign.

    Attributes:
        update (google.ads.googleads.v12.resources.types.SmartCampaignSetting):
            Update operation: The Smart campaign setting
            must specify a valid resource name.
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            FieldMask that determines which resource
            fields are modified in an update.
    """

    update = proto.Field(
        proto.MESSAGE,
        number=1,
        message=gagr_smart_campaign_setting.SmartCampaignSetting,
    )
    update_mask = proto.Field(
        proto.MESSAGE, number=2, message=field_mask_pb2.FieldMask,
    )


class MutateSmartCampaignSettingsResponse(proto.Message):
    r"""Response message for campaign mutate.

    Attributes:
        partial_failure_error (google.rpc.status_pb2.Status):
            Errors that pertain to operation failures in the partial
            failure mode. Returned only when partial_failure = true and
            all errors occur inside the operations. If any errors occur
            outside the operations (for example, auth errors), we return
            an RPC level error.
        results (Sequence[google.ads.googleads.v12.services.types.MutateSmartCampaignSettingResult]):
            All results for the mutate.
    """

    partial_failure_error = proto.Field(
        proto.MESSAGE, number=1, message=status_pb2.Status,
    )
    results = proto.RepeatedField(
        proto.MESSAGE, number=2, message="MutateSmartCampaignSettingResult",
    )


class MutateSmartCampaignSettingResult(proto.Message):
    r"""The result for the Smart campaign setting mutate.

    Attributes:
        resource_name (str):
            Returned for successful operations.
        smart_campaign_setting (google.ads.googleads.v12.resources.types.SmartCampaignSetting):
            The mutated Smart campaign setting with only mutable fields
            after mutate. The field will only be returned when
            response_content_type is set to "MUTABLE_RESOURCE".
    """

    resource_name = proto.Field(proto.STRING, number=1,)
    smart_campaign_setting = proto.Field(
        proto.MESSAGE,
        number=2,
        message=gagr_smart_campaign_setting.SmartCampaignSetting,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from airflow.providers.google_vendor.googleads.v12.common.types import custom_parameter
from airflow.providers.google_vendor.googleads.v12.common.types import feed_common
from airflow.providers.google_vendor.googleads.v12.enums.types import app_store as gage_app_store
from airflow.providers.google_vendor.googleads.v12.enums.types import (
    call_conversion_reporting_state as gage_call_conversion_reporting_state,
)
from airflow.providers.google_vendor.googleads.v12.enums.types import price_extension_price_qualifier
from airflow.providers.google_vendor.googleads.v12.enums.types import price_extension_price_unit
from airflow.providers.google_vendor.googleads.v12.enums.types import price_extension_type
from airflow.providers.google_vendor.googleads.v12.enums.types import (
    promotion_extension_discount_modifier,
)
from airflow.providers.google_vendor.googleads.v12.enums.types import promotion_extension_occasion


__protobuf__ = proto.module(
    package="airflow.providers.google_vendor.googleads.v12.common",
    marshal="google.ads.googleads.v12",
    manifest={
        "AppFeedItem",
        "CallFeedItem",
        "CalloutFeedItem",
        "LocationFeedItem",
        "AffiliateLocationFeedItem",
        "TextMessageFeedItem",
        "PriceFeedItem",
        "PriceOffer",
        "PromotionFeedItem",
        "StructuredSnippetFeedItem",
        "SitelinkFeedItem",
        "HotelCalloutFeedItem",
        "ImageFeedItem",
    },
)


class AppFeedItem(proto.Message):
    r"""Represents an App extension.

    Attributes:
        link_text (str):
            The visible text displayed when the link is
            rendered in an ad. This string must not be
            empty, and the length of this string should be
            between 1 and 25, inclusive.

            This field is a member of `oneof`_ ``_link_text``.
        app_id (str):
            The store-specific ID for the target
            application. This string must not be empty.

            This field is a member of `oneof`_ ``_app_id``.
        app_store (google.ads.googleads.v12.enums.types.AppStoreEnum.AppStore):
            The application store that the target
            application belongs to. This field is required.
        final_urls (Sequence[str]):
            A list of possible final URLs after all cross
            domain redirects. This list must not be empty.
        final_mobile_urls (Sequence[str]):
            A list of possible final mobile URLs after
            all cross domain redirects.
        tracking_url_template (str):
            URL template for constructing a tracking URL.
            Default value is "{lpurl}".

            This field is a member of `oneof`_ ``_tracking_url_template``.
        url_custom_parameters (Sequence[google.ads.googleads.v12.common.types.CustomParameter]):
            A list of mappings to be used for substituting URL custom
            parameter tags in the tracking_url_template, final_urls,
            and/or final_mobile_urls.
        final_url_suffix (str):
            URL template for appending params to landing
            page URLs served with parallel tracking.

            This field is a member of `oneof`_ ``_final_url_suffix``.
    """

    link_text = proto.Field(proto.STRING, number=9, optional=True,)
    app_id = proto.Field(proto.STRING, number=10, optional=True,)
    app_store = proto.Field(
        proto.ENUM, number=3, enum=gage_app_store.AppStoreEnum.AppStore,
    )
    final_urls = proto.RepeatedField(proto.STRING, number=11,)
    final_mobile_urls = proto.RepeatedField(proto.STRING, number=12,)
    tracking_url_template = proto.Field(proto.STRING, number=13, optional=True,)
    url_custom_parameters = proto.RepeatedField(
        proto.MESSAGE, number=7, message=custom_parameter.CustomParameter,
    )
    final_url_suffix = proto.Field(proto.STRING, number=14, optional=True,)


class CallFeedItem(proto.Message):
    r"""Represents a Call extension.

    Attributes:
        phone_number (str):
            The advertiser's phone number to append to
            the ad. This string must not be empty.

            This field is a member of `oneof`_ ``_phone_number``.
        country_code (str):
            Uppercase two-letter country code of the
            advertiser's phone number. This string must not
            be empty.

            This field is a member of `oneof`_ ``_country_code``.
        call_tracking_enabled (bool):
            Indicates whether call tracking is enabled.
            By default, call tracking is not enabled.

            This field is a member of `oneof`_ ``_call_tracking_enabled``.
        call_conversion_action (str):
            The conversion action to attribute a call conversion to. If
            not set a default conversion action is used. This field only
            has effect if call_tracking_enabled is set to true.
            Otherwise this field is ignored.

            This field is a member of `oneof`_ ``_call_conversion_action``.
        call_conversion_tracking_disabled (bool):
            If true, disable call conversion tracking.
            call_conversion_action should not be set if this is true.
            Optional.

            This field is a member of `oneof`_ ``_call_conversion_tracking_disabled``.
        call_conversion_reporting_state (google.ads.googleads.v12.enums.types.CallConversionReportingStateEnum.CallConversionReportingState):
            Enum value that indicates whether this call
            extension uses its own call conversion setting
            (or just have call conversion disabled), or
            following the account level setting.
    """

    phone_number = proto.Field(proto.STRING, number=7, optional=True,)
    country_code = proto.Field(proto.STRING, number=8, optional=True,)
    call_tracking_enabled = proto.Field(proto.BOOL, number=9, optional=True,)
    call_conversion_action = proto.Field(
        proto.STRING, number=10, optional=True,
    )
    call_conversion_tracking_disabled = proto.Field(
        proto.BOOL, number=11, optional=True,
    )
    call_conversion_reporting_state = proto.Field(
        proto.ENUM,
        number=6,
        enum=gage_call_conversion_reporting_state.CallConversionReportingStateEnum.CallConversionReportingState,
    )


class CalloutFeedItem(proto.Message):
    r"""Represents a callout extension.

    Attributes:
        callout_text (str):
            The callout text.
            The length of this string should be between 1
            and 25, inclusive.

            This field is a member of `oneof`_ ``_callout_text``.
    """

    callout_text = proto.Field(proto.STRING, number=2, optional=True,)


class LocationFeedItem(proto.Message):
    r"""Represents a location extension.

    Attributes:
        business_name (str):
            The name of the business.

            This field is a member of `oneof`_ ``_business_name``.
        address_line_1 (str):
            Line 1 of the business address.

            This field is a member of `oneof`_ ``_address_line_1``.
        address_line_2 (str):
            Line 2 of the business address.

            This field is a member of `oneof`_ ``_address_line_2``.
        city (str):
            City of the business address.

            This field is a member of `oneof`_ ``_city``.
        province (str):
            Province of the business address.

            This field is a member of `oneof`_ ``_province``.
        postal_code (str):
            Postal code of the business address.

            This field is a member of `oneof`_ ``_postal_code``.
        country_code (str):
            Country code of the business address.

            This field is a member of `oneof`_ ``_country_code``.
        phone_number (str):
            Phone number of the business.

            This field is a member of `oneof`_ ``_phone_number``.
    """

    business_name = proto.Field(proto.STRING, number=9, optional=True,)
    address_line_1 = proto.Field(proto.STRING, number=10, optional=True,)
    address_line_2 = proto.Field(proto.STRING, number=11, optional=True,)
    city = proto.Field(proto.STRING, number=12, optional=True,)
    province = proto.Field(proto.STRING, number=13, optional=True,)
    postal_code = proto.Field(proto.STRING, number=14, optional=True,)
    country_code = proto.Field(proto.STRING, number=15, optional=True,)
    phone_number = proto.Field(proto.STRING, number=16, optional=True,)


class AffiliateLocationFeedItem(proto.Message):
    r"""Represents an affiliate location extension.

    Attributes:
        business_name (str):
            The name of the business.

            This field is a member of `oneof`_ ``_business_name``.
        address_line_1 (str):
            Line 1 of the business address.

            This field is a member of `oneof`_ ``_address_line_1``.
        address_line_2 (str):
            Line 2 of the business address.

            This field is a member of `oneof`_ ``_address_line_2``.
        city (str):
            City of the business address.

            This field is a member of `oneof`_ ``_city``.
        province (str):
            Province of the business address.

            This field is a member of `oneof`_ ``_province``.
        postal_code (str):
            Postal code of the business address.

            This field is a member of `oneof`_ ``_postal_code``.
        country_code (str):
            Country code of the business address.

            This field is a member of `oneof`_ ``_country_code``.
        phone_number (str):
            Phone number of the business.

            This field is a member of `oneof`_ ``_phone_number``.
        chain_id (int):
            Id of the retail chain that is advertised as
            a seller of your product.

            This field is a member of `oneof`_ ``_chain_id``.
        chain_name (str):
            Name of chain.

            This field is a member of `oneof`_ ``_chain_name``.
    """

    business_name = proto.Field(proto.STRING, number=11, optional=True,)
    address_line_1 = proto.Field(proto.STRING, number=12, optional=True,)
    address_line_2 = proto.Field(proto.STRING, number=13, optional=True,)
    city = proto.Field(proto.STRING, number=14, optional=True,)
    province = proto.Field(proto.STRING, number=15, optional=True,)
    postal_code = proto.Field(proto.STRING, number=16, optional=True,)
    country_code = proto.Field(proto.STRING, number=17, optional=True,)
    phone_number = proto.Field(proto.STRING, number=18, optional=True,)
    chain_id = proto.Field(proto.INT64, number=19, optional=True,)
    chain_name = proto.Field(proto.STRING, number=20, optional=True,)


class TextMessageFeedItem(proto.Message):
    r"""An extension that users can click on to send a text message
    to the advertiser.

    Attributes:
        business_name (str):
            The business name to prepend to the message
            text. This field is required.

            This field is a member of `oneof`_ ``_business_name``.
        country_code (str):
            Uppercase two-letter country code of the
            advertiser's phone number. This field is
            required.

            This field is a member of `oneof`_ ``_country_code``.
        phone_number (str):
            The advertiser's phone number the message
            will be sent to. Required.

            This field is a member of `oneof`_ ``_phone_number``.
        text (str):
            The text to show in the ad.
            This field is required.

            This field is a member of `oneof`_ ``_text``.
        extension_text (str):
            The message extension_text populated in the messaging app.

            This field is a member of `oneof`_ ``_extension_text``.
    """

    business_name = proto.Field(proto.STRING, number=6, optional=True,)
    country_code = proto.Field(proto.STRING, number=7, optional=True,)
    phone_number = proto.Field(proto.STRING, number=8, optional=True,)
    text = proto.Field(proto.STRING, number=9, optional=True,)
    extension_text = proto.Field(proto.STRING, number=10, optional=True,)


class PriceFeedItem(proto.Message):
    r"""Represents a Price extension.

    Attributes:
        type_ (google.ads.googleads.v12.enums.types.PriceExtensionTypeEnum.PriceExtensionType):
            Price extension type of this extension.
        price_qualifier (google.ads.googleads.v12.enums.types.PriceExtensionPriceQualifierEnum.PriceExtensionPriceQualifier):
            Price qualifier for all offers of this price
            extension.
        tracking_url_template (str):
            Tracking URL template for all offers of this
            price extension.

            This field is a member of `oneof`_ ``_tracking_url_template``.
        language_code (str):
            The code of the language used for this price
            extension.

            This field is a member of `oneof`_ ``_language_code``.
        price_offerings (Sequence[google.ads.googleads.v12.common.types.PriceOffer]):
            The price offerings in this price extension.
        final_url_suffix (str):
            Tracking URL template for all offers of this
            price extension.

            This field is a member of `oneof`_ ``_final_url_suffix``.
    """

    type_ = proto.Field(
        proto.ENUM,
        number=1,
        enum=price_extension_type.PriceExtensionTypeEnum.PriceExtensionType,
    )
    price_qualifier = proto.Field(
        proto.ENUM,
        number=2,
        enum=price_extension_price_qualifier.PriceExtensionPriceQualifierEnum.PriceExtensionPriceQualifier,
    )
    tracking_url_template = proto.Field(proto.STRING, number=7, optional=True,)
    language_code = proto.Field(proto.STRING, number=8, optional=True,)
    price_offerings = proto.RepeatedField(
        proto.MESSAGE, number=5, message="PriceOffer",
    )
    final_url_suffix = proto.Field(proto.STRING, number=9, optional=True,)


class PriceOffer(proto.Message):
    r"""Represents one price offer in a price extension.

    Attributes:
        header (str):
            Header text of this offer.

            This field is a member of `oneof`_ ``_header``.
        description (str):
            Description text of this offer.

            This field is a member of `oneof`_ ``_description``.
        price (google.ads.googleads.v12.common.types.Money):
            Price value of this offer.
        unit (google.ads.googleads.v12.enums.types.PriceExtensionPriceUnitEnum.PriceExtensionPriceUnit):
            Price unit for this offer.
        final_urls (Sequence[str]):
            A list of possible final URLs after all cross
            domain redirects.
        final_mobile_urls (Sequence[str]):
            A list of possible final mobile URLs after
            all cross domain redirects.
    """

    header = proto.Field(proto.STRING, number=7, optional=True,)
    description = proto.Field(proto.STRING, number=8, optional=True,)
    price = proto.Field(proto.MESSAGE, number=3, message=feed_common.Money,)
    unit = proto.Field(
        proto.ENUM,
        number=4,
        enum=price_extension_price_unit.PriceExtensionPriceUnitEnum.PriceExtensionPriceUnit,
    )
    final_urls = proto.RepeatedField(proto.STRING, number=9,)
    final_mobile_urls = proto.RepeatedField(proto.STRING, number=10,)


class PromotionFeedItem(proto.Message):
    r"""Represents a Promotion extension.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        promotion_target (str):
            A freeform description of what the promotion
            is targeting. This field is required.

            This field is a member of `oneof`_ ``_promotion_target``.
        discount_modifier (google.ads.googleads.v12.enums.types.PromotionExtensionDiscountModifierEnum.PromotionExtensionDiscountModifier):
            Enum that modifies the qualification of the
            discount.
        promotion_start_date (str):
            Start date of when the promotion is eligible
            to be redeemed.

            This field is a member of `oneof`_ ``_promotion_start_date``.
        promotion_end_date (str):
            Last date when the promotion is eligible to
            be redeemed.

            This field is a member of `oneof`_ ``_promotion_end_date``.
        occasion (google.ads.googleads.v12.enums.types.PromotionExtensionOccasionEnum.PromotionExtensionOccasion):
            The occasion the promotion was intended for.
            If an occasion is set, the redemption window
            will need to fall within the date range
            associated with the occasion.
        final_urls (Sequence[str]):
            A list of possible final URLs after all cross
            domain redirects. This field is required.
        final_mobile_urls (Sequence[str]):
            A list of possible final mobile URLs after
            all cross domain redirects.
        tracking_url_template (str):
            URL template for constructing a tracking URL.

            This field is a member of `oneof`_ ``_tracking_url_template``.
        url_custom_parameters (Sequence[google.ads.googleads.v12.common.types.CustomParameter]):
            A list of mappings to be used for substituting URL custom
            parameter tags in the tracking_url_template, final_urls,
            and/or final_mobile_urls.
        final_url_suffix (str):
            URL template for appending params to landing
            page URLs served with parallel tracking.

            This field is a member of `oneof`_ ``_final_url_suffix``.
        language_code (str):
            The language of the promotion.
            Represented as BCP 47 language tag.

            This field is a member of `oneof`_ ``_language_code``.
        percent_off (int):
            Percentage off discount in the promotion in micros. One
            million is equivalent to one percent. Either this or
            money_off_amount is required.

            This field is a member of `oneof`_ ``discount_type``.
        money_amount_off (google.ads.googleads.v12.common.types.Money):
            Money amount off for discount in the promotion. Either this
            or percent_off is required.

            This field is a member of `oneof`_ ``discount_type``.
        promotion_code (str):
            A code the user should use in order to be
            eligible for the promotion.

            This field is a member of `oneof`_ ``promotion_trigger``.
        orders_over_amount (google.ads.googleads.v12.common.types.Money):
            The amount the total order needs to be for
            the user to be eligible for the promotion.

            This field is a member of `oneof`_ ``promotion_trigger``.
    """

    promotion_target = proto.Field(proto.STRING, number=16, optional=True,)
    discount_modifier = proto.Field(
        proto.ENUM,
        number=2,
        enum=promotion_extension_discount_modifier.PromotionExtensionDiscountModifierEnum.PromotionExtensionDiscountModifier,
    )
    promotion_start_date = proto.Field(proto.STRING, number=19, optional=True,)
    promotion_end_date = proto.Field(proto.STRING, number=20, optional=True,)
    occasion = proto.Field(
        proto.ENUM,
        number=9,
        enum=promotion_extension_occasion.PromotionExtensionOccasionEnum.PromotionExtensionOccasion,
    )
    final_urls = proto.RepeatedField(proto.STRING, number=21,)
    final_mobile_urls = proto.RepeatedField(proto.STRING, number=22,)
    tracking_url_template = proto.Field(proto.STRING, number=23, optional=True,)
    url_custom_parameters = proto.RepeatedField(
        proto.MESSAGE, number=13, message=custom_parameter.CustomParameter,
    )
    final_url_suffix = proto.Field(proto.STRING, number=24, optional=True,)
    language_code = proto.Field(proto.STRING, number=25, optional=True,)
    percent_off = proto.Field(proto.INT64, number=17, oneof="discount_type",)
    money_amount_off = proto.Field(
        proto.MESSAGE,
        number=4,
        oneof="discount_type",
        message=feed_common.Money,
    )
    promotion_code = proto.Field(
        proto.STRING, number=18, oneof="promotion_trigger",
    )
    orders_over_amount = proto.Field(
        proto.MESSAGE,
        number=6,
        oneof="promotion_trigger",
        message=feed_common.Money,
    )


class StructuredSnippetFeedItem(proto.Message):
    r"""Represents a structured snippet extension.

    Attributes:
        header (str):
            The header of the snippet.
            This string must not be empty.

            This field is a member of `oneof`_ ``_header``.
        values (Sequence[str]):
            The values in the snippet.
            The maximum size of this collection is 10.
    """

    header = proto.Field(proto.STRING, number=3, optional=True,)
    values = proto.RepeatedField(proto.STRING, number=4,)


class SitelinkFeedItem(proto.Message):
    r"""Represents a sitelink extension.

    Attributes:
        link_text (str):
            URL display text for the sitelink.
            The length of this string should be between 1
            and 25, inclusive.

            This field is a member of `oneof`_ ``_link_text``.
        line1 (str):
            First line of the description for the
            sitelink. If this value is set, line2 must also
            be set. The length of this string should be
            between 0 and 35, inclusive.

            This field is a member of `oneof`_ ``_line1``.
        line2 (str):
            Second line of the description for the
            sitelink. If this value is set, line1 must also
            be set. The length of this string should be
            between 0 and 35, inclusive.

            This field is a member of `oneof`_ ``_line2``.
        final_urls (Sequence[str]):
            A list of possible final URLs after all cross
            domain redirects.
        final_mobile_urls (Sequence[str]):
            A list of possible final mobile URLs after
            all cross domain redirects.
        tracking_url_template (str):
            URL template for constructing a tracking URL.

            This field is a member of `oneof`_ ``_tracking_url_template``.
        url_custom_parameters (Sequence[google.ads.googleads.v12.common.types.CustomParameter]):
            A list of mappings to be used for substituting URL custom
            parameter tags in the tracking_url_template, final_urls,
            and/or final_mobile_urls.
        final_url_suffix (str):
            Final URL suffix to be appended to landing
            page URLs served with parallel tracking.

            This field is a member of `oneof`_ ``_final_url_suffix``.
    """

    link_text = proto.Field(proto.STRING, number=9, optional=True,)
    line1 = proto.Field(proto.STRING, number=10, optional=True,)
    line2 = proto.Field(proto.STRING, number=11, optional=True,)
    final_urls = proto.RepeatedField(proto.STRING, number=12,)
    final_mobile_urls = proto.RepeatedField(proto.STRING, number=13,)
    tracking_url_template = proto.Field(proto.STRING, number=14, optional=True,)
    url_custom_parameters = proto.RepeatedField(
        proto.MESSAGE, number=7, message=custom_parameter.CustomParameter,
    )
    final_url_suffix = proto.Field(proto.STRING, number=15, optional=True,)


class HotelCalloutFeedItem(proto.Message):
    r"""Represents a hotel callout extension.

    Attributes:
        text (str):
            The callout text.
            The length of this string should be between 1
            and 25, inclusive.

            This field is a member of `oneof`_ ``_text``.
        language_code (str):
            The language of the hotel callout text.
            IETF BCP 47 compliant language code.

            This field is a member of `oneof`_ ``_language_code``.
    """

    text = proto.Field(proto.STRING, number=3, optional=True,)
    language_code = proto.Field(proto.STRING, number=4, optional=True,)


class ImageFeedItem(proto.Message):
    r"""Represents an advertiser provided image extension.

    Attributes:
        image_asset (str):
            Required. Resource name of the image asset.
    """

    image_asset = proto.Field(proto.STRING, number=1,)


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from airflow.providers.google_vendor.googleads.v12.common.types import (
    policy_summary as gagc_policy_summary,
)
from airflow.providers.google_vendor.googleads.v12.enums.types import asset_field_type
from airflow.providers.google_vendor.googleads.v12.enums.types import asset_link_status
from airflow.providers.google_vendor.googleads.v12.enums.types import asset_performance_label


__protobuf__ = proto.module(
    package="airflow.providers.google_vendor.googleads.v12.resources",
    marshal="google.ads.googleads.v12",
    manifest={"AssetGroupAsset",},
)


class AssetGroupAsset(proto.Message):
    r"""AssetGroupAsset is the link between an asset and an asset
    group. Adding an AssetGroupAsset links an asset with an asset
    group.

    Attributes:
        resource_name (str):
            Immutable. The resource name of the asset group asset. Asset
            group asset resource name have the form:

            ``customers/{customer_id}/assetGroupAssets/{asset_group_id}~{asset_id}~{field_type}``
        asset_group (str):
            Immutable. The asset group which this asset
            group asset is linking.
        asset (str):
            Immutable. The asset which this asset group
            asset is linking.
        field_type (google.ads.googleads.v12.enums.types.AssetFieldTypeEnum.AssetFieldType):
            The description of the placement of the asset within the
            asset group. For example: HEADLINE, YOUTUBE_VIDEO etc
        status (google.ads.googleads.v12.enums.types.AssetLinkStatusEnum.AssetLinkStatus):
            The status of the link between an asset and
            asset group.
        performance_label (google.ads.googleads.v12.enums.types.AssetPerformanceLabelEnum.AssetPerformanceLabel):
            Output only. The performance of this asset
            group asset.
        policy_summary (google.ads.googleads.v12.common.types.PolicySummary):
            Output only. The policy information for this
            asset group asset.
    """

    resource_name = proto.Field(proto.STRING, number=1,)
    asset_group = proto.Field(proto.STRING, number=2,)
    asset = proto.Field(proto.STRING, number=3,)
    field_type = proto.Field(
        proto.ENUM,
        number=4,
        enum=asset_field_type.AssetFieldTypeEnum.AssetFieldType,
    )
    status = proto.Field(
        proto.ENUM,
        number=5,
        enum=asset_link_status.AssetLinkStatusEnum.AssetLinkStatus,
    )
    performance_label = proto.Field(
        proto.ENUM,
        number=6,
        enum=asset_performance_label.AssetPerformanceLabelEnum.AssetPerformanceLabel,
    )
    policy_summary = proto.Field(
        proto.MESSAGE, number=7, message=gagc_policy_summary.PolicySummary,
    )


__all__ = tuple(sorted(__protobuf__.manifest))
